/**
 * Pollings component
 *
 *
 * @namespace Alfresco
 * @class Alfresco.Pollings
 */

(function()
{

    /**
     * YUI Library aliases
     */
    var Dom = YAHOO.util.Dom;
         
    /**
     * Alfresco Slingshot aliases
     */
    var $html = Alfresco.util.encodeHTML;

    /**
     * Pollings constructor.
     *
     * @param {String} htmlId The HTML id of the parent element
     * @return {Alfresco.SiteFinder} The new SiteFinder instance
     * @constructor
     */
    Alfresco.Pollings = function(htmlId)
    {
        Alfresco.Pollings.superclass.constructor.call(this, "Alfresco.Pollings", htmlId, ["button", "container", "datasource", "datatable", "paginator", "json"]);
        YAHOO.Bubbling.on("changeFilter", this.onChangeFilter, this);
    };

    YAHOO.extend(Alfresco.Pollings, Alfresco.component.Base,
    {
   
        /**
         * The deleted link CSS style.
         */
        DELETEDCLASS: "delete-polling",

        /**
         * The edited link CSS style.
         */
        EDITEDCLASS: "edit-polling",
		
		
		RESULTSCLASS: "view-results",
   
        /**
         * Object literal used to generate unique tag ids
         *
         * @property tagId
         * @type object
         */
        tagId: null,

        /**
         * Tells whether an action is currently ongoing.
         *
         * @property busy
         * @type boolean
         * @see _setBusy/_releaseBusy
         */
        busy: null,

        /**
         * Offset of first record on page
         * 
         * @property recordOffset
         * @type int
         * @default 0
         */
        recordOffset: 0,

        /**
         * Total number of posts in the current view (across all pages)
         * 
         * @property totalRecords
         * @type int
         * @default 0
         */
        totalRecords: 0,
      
        /**
         * Object container for initialization options
         *
         * @property options
         * @type object
         */
        options:
        {
            /**
             * Current siteId.
             *
             * @property siteId
             * @type string
             */
            siteId: "",

            /**
             * Initially used filter name and id.
             */
            initialFilter: {},

            /**
             * Number of items displayed per page
             *
             * @property pageSize
             * @type int
             */
            pageSize: 10,

            /**
             * Flag indicating whether the list shows a detailed view or a simple one.
             *
             * @property simpleView
             * @type boolean
             */
            simpleView: false,

            /**
             * permission delete
             */
            permissionDelete: true,

            /**
             * permission update
             */
            permissionUpdate:true,

            /**
             * Minimal length of filter panel
             */
            MIN_FILTER_PANEL_WIDTH: 150,

            /**
             * Maximal length of filter panel
             */
            MAX_FILTER_PANEL_WIDTH: 640 - ((YAHOO.env.ua.ie > 0) && (YAHOO.env.ua.ie < 7) ? 160 : 0),

            /**
             * The pagination flag.
             *
             * @property: usePagination
             * @type: boolean
             * @default: true
             */
            usePagination: true,

            /**
             * Minimal height of filter panel
             */
            MAX_FILTER_PANEL_HEIGHT: 200,

            /**
             * ContainerId representing root container
             *
             * @property containerId
             * @type string
             */
            containerId: ""
        },

        /**
         * Fired by YUI when parent element is available for scripting.
         * Component initialisation, including instantiation of YUI widgets and event listener binding.
         *
         * @method onReady
         */
        onReady: function Pollings_onReady()
        {
            this.activate();
        },

        /**
         * activation of components
         * @method activate.
         */
        activate: function Pollings_activate()
        {
            this.attachButtons();
            Dom.setStyle(this.id + '-pollings-header', 'visibility', 'visible');
            Dom.setStyle(this.id + '-body', 'visibility', 'visible');
            this.createDataSource();
            this.createDataTable();
        },
	  
        /**
         * init DataSource
         * @method createDataSource
         */
        createDataSource: function Pollings_createDataSource()
        {
            var uriResults = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/pollings/site/{site}/{container}",
            {
                site: this.options.siteId,
                container: this.options.containerId
            });

            this.widgets.dataSource = new YAHOO.util.DataSource(uriResults,
            {
                responseType: YAHOO.util.DataSource.TYPE_JSON,
                connXhrMode: 'queueRequests',
                responseSchema:
                {
                    resultsList: 'items',
                    metaFields:
                    {
                        recordOffset: 'startIndex',
                        totalRecords: 'total',
                        metadata: 'metadata'
                    }
                }
            });

        },

        /**
         * init pollings buttons
         * @method attachButtons.
         */
        attachButtons: function Pollings_attachButtons()
        {
            this.widgets.newPollingBtn = Alfresco.util.createYUIButton(this, "create-polling-button", this.showCreatePollingDlg,
            {
                disabled: false,
                value: "create"
            });
            this.widgets.pollingsSelectMenu = Alfresco.util.createYUIButton(this, "select-button", this.onSelectItemClick,
            {
                type: "menu",
                menu: "selectItems-menu"
            });
        },
	  
        onSelectItemClick: function Pollings_onSelectItemClick(sType, aArgs, p_obj)
        {
            var elem = YAHOO.env.ua.ie ? aArgs[0].srcElement : aArgs[0].target;
            if (elem.tagName.toLocaleLowerCase() != "span")
            {
                elem = elem.getElementsByTagName("span")[0];
            }
            switch (elem.className.split(" ")[0])
            {
                case "pollings-action-deselect-all":
                    //this.deselectAll();
                    //this.widgets.pollingsMenu.set("disabled", true);
                    break;

                case "pollings-action-select-all":
                    //this.selectAll();
                    //this.widgets.pollingsMenu.set("disabled", !this.getSelectedPollings().length);
                    break;

                case "pollings-action-invert-selection":
                    //this.invertAll();
                    break;
            }
        },

        /**
         * show 'Create Polling' dialog
         * @method showCreateLinkDlg.
         */
        showCreatePollingDlg: function Pollings_showCreatePollingDlg()
        {
            var url = YAHOO.lang.substitute(Alfresco.constants.URL_PAGECONTEXT + "site/{site}/pollings-pollingedit",
            {
                site: this.options.siteId
            });
            window.location = url;
        },
	  
        /**
         * Gets the array of selected pollings.
         *
         * @method getSelectedPollings
         */
        getSelectedPollings: function Pollings_getSelectedPollings()
        {
            var arr = [];
            var rows = this.widgets.dataTable.getTbodyEl().rows;
            for (var i = 0; i < rows.length; i++)
            {
                if (rows[i].cells[0].getElementsByTagName('input')[0].checked)
                {
                    var data = this.widgets.dataTable.getRecord(i);
                    if (data)
                    {
                        arr.push(data);
                    }
                }
            }

            return arr;
        },

        /**
         * Pollings Filter changed event handler
         *
         * @method onChangeFilter
         * @param layer {object} Event fired (unused)
         * @param args {array} Event parameters (new filterId)
         */
        onChangeFilter: function Pollings_onChangeFilter(layer, args)
        {
            var obj = args[1];
            if ((obj !== null) && (obj.filterId !== null))
            {
                this.currentFilter =
                {
                    filterId: obj.filterId,
                    filterOwner: obj.filterOwner,
                    filterData: obj.filterData
                };
                this.updatePollings(
                {
                    page: 1
                });
                YAHOO.Bubbling.fire("filterChanged", this.currentFilter);
            }
        },
        
        createDataTable: function Pollings_createDataTable()
        {
            var me = this;

            /**
             * Selector custom datacell formatter
             *
             * @method renderCellSelected
             * @param elCell {object}
             * @param oRecord {object}
             * @param oColumn {object}
             * @param oData {object|string}
             */
            var renderCellSelected = function Pollings_renderCellSelected(elCell, oRecord, oColumn, oData)
            {
                elCell.innerHTML = '<input class="checkbox-column" type="checkbox" />';
                elCell.firstChild.onclick = function()
                {
                    // [TODO]
                    //me.widgets.pollingsMenu.set("disabled", me.getSelectedPollings().length === 0);
                };
            };

            /**
             * Description custom datacell formatter
             *
             * @method renderCellDescription
             * @param elCell {object}
             * @param oRecord {object}
             * @param oColumn {object}
             * @param oData {object|string}
             */
            var renderCellDescription = function Pollings_renderCellDescription(elCell, oRecord, oColumn, oData)
            {
                var data = oRecord.getData();
                var title = data.title,
                description = data.description,
                createdOn = data.createdOn,
                author = data.author,
                tags = data.tags;
            
                var pollingPostUrl = me.generatePollingPostUrl(me.options.siteId, data.nodeRef, data.pollingItemType);
			
                var innerHtml = '<h3 class="polling-title"><a href="' + pollingPostUrl + '" class="theme-color-1">' + $html(title) + '</a></h3>';
                // [TODO] simple view
            
                if (!me.options.simpleView)
                {
                    var tagsStr = "";
                    if (tags.length > 0)
                    {
                        for (var i = 0; i < tags.length; i++)
                        {
                            tagsStr += me._generateTagLink(tags[i]);
                            if (i != (tags.length - 1))
                            {
                                tagsStr += ', &nbsp;';
                            }
                        }
                    }
                    else
                    {
                        tagsStr = me.msg("details.tags.none");
                    }
			
                    innerHtml += '<div class="detail"><span class="item"><em>' + me.msg("details.created.on") + ':</em> ' + Alfresco.util.formatDate(createdOn) + '</span>' +
                    '<span class="item"><em>' + me.msg("details.created.by") + ':</em> ' + Alfresco.util.people.generateUserLink(author) + '</span></div>';
                    innerHtml += '<div class="detail"><span class="item"><em>' + me.msg("details.description") + ':</em> ' + $html(description) + '</span></div>';
                    innerHtml += '<div class="detail"><span class="tag-item"><em>' + me.msg("details.tags") + ': </em>' + tagsStr + '</span></div>';
                }

                elCell.innerHTML = innerHtml;
            };
            
            /**
             * Actions custom datacell formatter
             *
             * @method renderCellActions
             * @param elCell {object}
             * @param oRecord {object}
             * @param oColumn {object}
             * @param oData {object|string}
             */
            var renderCellActions = function Pollings_renderCellActions(elCell, oRecord, oColumn, oData)
            {
                var permissions = oRecord.getData("permissions");
               
                elCell.style.visibility = "hidden";
                elCell.innerHTML =
					"<div class='" + me.EDITEDCLASS +  "'><a><span class='theme-color-1'>" + me.msg("pollings.edit") + "</a></span></div>" +
					"<div class='" + me.DELETEDCLASS + "'><a><span class='theme-color-1'>" + me.msg("pollings.delete") + "</a></span></div>" + 
					"<div class='" + me.RESULTSCLASS + "'><a><span class='theme-color-1'>" + me.msg("pollings.results") + "</a></span></div>";

                var ec = elCell.childNodes[0],
                    dc = elCell.childNodes[1],
					rc = elCell.childNodes[2];

                // Edit permission?
                if (permissions.edit)
                {
                    ec.onclick = function Pollings_onEditPolling()
                    {
                        window.location = YAHOO.lang.substitute(Alfresco.constants.URL_PAGECONTEXT + "site/{site}/pollings-pollingedit?pollingId={polling}",
                        {
                            site: me.options.siteId,
                            polling: oRecord.getData('name')
                        });
                    };

                    ec.onmouseover = function()
                    {
                        Dom.addClass(this, me.EDITEDCLASS + "-over");
                    };

                    ec.onmouseout = function()
                    {
                        Dom.removeClass(this, me.EDITEDCLASS + "-over");
                    };
                }
                else
                {
                    Dom.addClass(ec, 'hidden');
                }
            
                // Delete permission?
                if (permissions["delete"])
                {
                    dc.onclick = function ()
                    {
                    // [TODO]
                    /*
                  var mes = me.msg("dialog.confirm.message.delete", title);
                  var callback = function()
                  {
                     me.deleteLinks([oRecord]);
                  };
                  me.showConfirmDialog(mes, callback);
				  */
                    };

                    dc.onmouseover = function()
                    {
                        Dom.addClass(this, me.DELETEDCLASS + "-over");
                    };

                    dc.onmouseout = function()
                    {
                        Dom.removeClass(this, me.DELETEDCLASS + "-over");
                    };

                }
                else
                {
                    Dom.addClass(dc, 'hidden');
                }
				
				if (permissions.edit)
				{
                    rc.onclick = function Pollings_onViewResults()
                    {
                        window.location = YAHOO.lang.substitute(Alfresco.constants.URL_PAGECONTEXT + "site/{site}/pollings-results?pollingId={polling}",
                        {
                            site: me.options.siteId,
                            polling: oRecord.getData('name')
                        });
                    };
                    rc.onmouseover = function()
                    {
                        Dom.addClass(this, me.RESULTSCLASS + "-over");
                    };
                    rc.onmouseout = function()
                    {
                        Dom.removeClass(this, me.RESULTSCLASS + "-over");
                    };
				}
				
                // Styling
                Dom.setStyle(elCell.parentNode, "border-left", "3px solid #fff");
                if (me.options.simpleView)
                {
                    Dom.addClass(elCell.parentNode, 'simple-view');
                }
                else
                {
                    Dom.removeClass(elCell.parentNode, 'simple-view');
                }
            };
         
            var columnDefinitions =
            [
            {
                key: 'selected', 
                label: 'Selected', 
                sortable: false, 
                formatter: renderCellSelected
            },
            {
                key: 'title', 
                label: 'Title', 
                sortable: false, 
                formatter: renderCellDescription
            },
            {
                key: 'description', 
                label: 'Description', 
                formatter: renderCellActions
            }
            ];

            this.widgets.paginator = new YAHOO.widget.Paginator(
            {
                containers: [this.id + "-paginator"],
                rowsPerPage: this.options.pageSize,
                initialPage: 1,
                template: this.msg("pagination.template"),
                pageReportTemplate: this.msg("pagination.template.page-report"),
                previousPageLinkLabel: this.msg("pagination.previousPageLinkLabel"),
                nextPageLinkLabel: this.msg("pagination.nextPageLinkLabel")
            });

            // called by the paginator on state changes
            var handlePagination = function Pollings_handlePagination (state, dt)
            {
                me.updatePollings(
                {
                    page: state.page
                });
            };

            this.widgets.paginator.subscribe("changeRequest", handlePagination);

            this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + '-pollings', columnDefinitions, this.widgets.dataSource,
            {
                renderLoopSize: 32,
                initialLoad: false,
                MSG_EMPTY: '<span class="datatable-msg-empty">' + this.msg("pollings.empty") + '</span>'
            });

            YAHOO.widget.DataTable.CLASS_SELECTED = "pollings-selected-row";

            this.widgets.dataTable.doBeforeLoadData = function Pollings_doBeforeLoadData(sRequest, oResponse, oPayload)
            {
                if (oResponse.error)
                {
                    try
                    {
                        var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                        me.widgets.dataTable.set("MSG_ERROR", response.message);
                    }
                    catch(e)
                    {
                    }

                }
                else if (oResponse.results && !me.options.usePagination)
                {
                    this.renderLoopSize = Alfresco.util.RENDERLOOPSIZE;
                }

                // Must return true to have the "Loading..." message replaced by the error message
                return true;
            };

            // Update totalRecords on the fly with value from server
            this.widgets.dataTable.handleDataReturnPayload = function Pollings_handleDataReturnPayload(oRequest, oResponse, oPayload)
            {
                // Save totalRecords for Paginator update later
                me.recordOffset = oResponse.meta.recordOffset;
                me.totalRecords = oResponse.meta.totalRecords;

                oPayload = oPayload || {};
                oPayload.recordOffset = oResponse.meta.recordOffset;
                oPayload.totalRecords = oResponse.meta.totalRecords;
                return oPayload;
            };

            // Prevent the DataTable from updating the Paginator widget
            this.widgets.dataTable.doBeforePaginatorChange = function Pollings_doBeforePaginatorChange(oPaginatorState)
            {
                return false;
            };

            // Rendering complete event handler
            this.widgets.dataTable.subscribe("renderEvent", function()
            {
                // Update the paginator if it's been created
                this.widgets.paginator.setState(
                {
                    recordOffset: this.recordOffset,
                    totalRecords: this.totalRecords
                });
                this.widgets.paginator.render();
            }, this, true);

            this.widgets.dataTable.subscribe("tableMsgShowEvent", function(oArgs)
            {
                // NOTE: Scope needs to be DataTable
                this._elMsgTbody.parentNode.style.width = "";
            });

            this.widgets.dataTable.set("selectionMode", "single");

            var onRowMouseover = function Pollings_onRowMouseover(e)
            {
                me.widgets.dataTable.selectRow(e.target);
                e.target.cells[2].childNodes[0].style.visibility = "visible";
                e.target.cells[2].childNodes[0].style.width = "100px";
                e.target.cells[2].style.borderLeft = "1px solid #999";
            };

            var onRowMouseout = function Pollings_onRowMouseout(e)
            {
                me.widgets.dataTable.unselectRow(e.target);
                e.target.cells[2].childNodes[0].style.visibility = "hidden";
                e.target.cells[2].style.borderLeft = "1px solid #FFF";
            };

            this.widgets.dataTable.subscribe("rowMouseoverEvent", onRowMouseover);
            this.widgets.dataTable.subscribe("rowMouseoutEvent", onRowMouseout);

            var filterObj = YAHOO.lang.merge(
            {
                filterId: "all",
                filterOwner: "Alfresco.PollingFilter",
                filterData: null
            }, this.options.initialFilter);
         
            YAHOO.Bubbling.fire("changeFilter", filterObj);

        },

        /**
         * Updates pollings list by calling data webscript with current site and filter information
         *
         * @method updatePollings
         */
        updatePollings: function Pollings_updatePollings(p_obj)
        {
            function successHandler(sRequest, oResponse, oPayload)
            {
                this.widgets.dataTable.onDataReturnInitializeTable.call(this.widgets.dataTable, sRequest, oResponse, oPayload);
                // [TODO]
                // this.updateListTitle();
                // this.widgets.linksMenu.set("disabled", this.getSelectedLinks().length === 0);
                var perm = oResponse.meta.metadata.pollingPermissions;
                this.options.permissionDelete = perm["delete"];
                this.options.permissionUpdate = perm.edit;
                 // [TODO]
                // this.updateToolbar(perm);
            }

            function failureHandler(sRequest, oResponse)
            {
                if (oResponse.status == 401)
                {
                    // Our session has likely timed-out, so refresh to offer the login page
                    window.location.reload(true);
                }
                else
                {
                    try
                    {
                        var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                        this.widgets.dataTable.set("MSG_ERROR", response.message);
                        this.widgets.dataTable.showTableMessage(response.message, YAHOO.widget.DataTable.CLASS_ERROR);
                        if (oResponse.status == 404)
                        {
                            YAHOO.Bubbling.fire("deactivateAllControls");
                        }
                    }
                    catch(e)
                    {
                    }
                }
            }
            this.widgets.dataSource.sendRequest(this._buildPollingsParams(p_obj || {}),
            {
                success: successHandler,
                failure: failureHandler,
                scope: this
            });
        },

        generatePollingPostUrl: function Pollings_generatePollingPostUrl(site, pollingId, pollingItemType)
        {
            var url = YAHOO.lang.substitute(Alfresco.constants.URL_PAGECONTEXT + "site/{site}/pollings-post?destination={pollingId}&itemId={itemType}",
            {
                site: site,
                pollingId: pollingId,
                itemType: pollingItemType
            });
            return url;
        },
        
        /**
         * PRIVATE FUNCTIONS
         */
	   
        /**
         * Generate the html markup for a tag link.
         *
         * @method _generateTagLink
         * @param tagName {string} the tag to create a link for
         * @return {string} the markup for a tag
         */
        _generateTagLink: function Pollings_generateTagLink(tagName)
        {
            var encodedTagName = $html(tagName);
            return '<span class="tag"><a href="#" class="tag-link" rel="' + encodedTagName + '" title="' + encodedTagName + '">' + encodedTagName + '</a></span>';
        },

        /** Build URI parameter string for pollings JSON data webscript
         *
         * @method _buildPollingsParams
         * @param p_obj.page {string} Page number
         * @param p_obj.pageSize {string} Number of items per page
         */
        _buildPollingsParams: function Pollings_buildPollingsParams(p_obj)
        {
            var params =
            {
                tag: null,
                page: this.widgets.paginator.getCurrentPage() || "1",
                pageSize: this.widgets.paginator.getRowsPerPage()
            };

            // Passed-in overrides
            if (typeof p_obj == "object")
            {
                params = YAHOO.lang.merge(params, p_obj);
            }

            // calculate the startIndex param
            params.startIndex = (params.page - 1) * params.pageSize;

            // check what url to call and with what parameters
            var filterOwner = this.currentFilter.filterOwner;
            var filterId = this.currentFilter.filterId;
            var filterData = this.currentFilter.filterData;

            // check whether we got a filter or not
            var url = "";
            if (filterOwner == "Alfresco.PollingFilter")
            {
                url = "?filter=" + filterId;
            }
            else if (filterOwner == "Alfresco.TagFilter")
            {
                url = "?filter=tag";
                params.tag = filterData;
            }

            // build the url extension
            var urlExt = "";
            for (var paramName in params)
            {
                if (params[paramName] !== null)
                {
                    urlExt += "&" + paramName + "=" + encodeURIComponent(params[paramName]);
                }
            }
            if (urlExt.length > 0)
            {
                urlExt = urlExt.substring(1);
            }
            return url + "&" + urlExt;
        }
      
    });
})();